extends Node #Audio autoload scene


const c_VOLUMES := [
	-80.0, -28.0, -24.0, -20.0, -16.0, 
	-12.0, -8.0, -6.0, -3.0, 0.0
]

var m_sfx := {}
var m_sfx_volume := 0.0

func _ready():
	var sfx_dir := "res://Audio/Sfx/"
	var music_dir := "res://Audio/Music/"
	
	var sfx : Array = get_dir_files(sfx_dir)
	for file in sfx:
		if ".import" in file:
			#if !OS.is_debug_build():
			file = file.replace(".import","")
			#else:
			#	continue
		#print("Globals::_ready: " + file)
		var key : String = file.split('.')[0]
		if m_sfx.has(key):
			continue
		m_sfx[key] = load(sfx_dir + file)
		var sound := AudioStreamPlayer.new()
		sound.stream= m_sfx[key]
		$Sfx.add_child(sound)
		sound.name = key
		sfx(key, -60.0)
	
	for sfx_child in $Sfx.get_children():
		sfx_child.stop()

func sfx(name : String, vol : float = 0.0, bus : String = "sfx"):
	#print("Audio::sfx: " + name + ", " + str(vol) + "db")
	var sfx : AudioStreamPlayer = $Sfx.get_node(name)
	if sfx == null:
		return
	sfx.bus = bus
	assert(sfx.bus == bus)
	sfx.volume_db = vol
	sfx.play()

func is_sfx_playing(name : String)->bool:
	return $Sfx.get_node(name).playing

func sfx_looped(name : String, vol : float = 0.0, bus : String = "sfx"):
	if !is_sfx_playing(name):
		sfx(name, vol, bus)

func sfx_stop(name : String):
	var sfx : AudioStreamPlayer = $Sfx.get_node(name)
	if sfx == null:
		return
	sfx.stop()

func sfx_deferred(name : String, time : float, vol : float = 0.0, bus : String = "sfx"):
	yield(get_tree().create_timer(time), "timeout")
	sfx(name, vol, bus)

func cache_audio():
	for sfx in $Sfx.get_children():
		sfx(sfx.name, -60.0)


func set_master_vol(idx : int):	
	_set_bus_vol("Master", idx)

func set_music_vol(idx : int):
	_set_bus_vol("music", idx)

func set_sfx_vol(idx : int):
	_set_bus_vol("sfx_group", idx)
	
func _set_bus_vol(bus : String, idx : int):
	var bus_idx : int = AudioServer.get_bus_index(bus)
	AudioServer.set_bus_volume_db(bus_idx, c_VOLUMES[idx])

func get_dir_files(directory_path : String )->Array:
	var file_names := []
	var dir = Directory.new()
	if dir.open(directory_path) == OK:
		dir.list_dir_begin()
		var file_name = dir.get_next()
		while file_name != "":
			if !dir.current_is_dir():
				file_names.push_back(file_name)
			file_name = dir.get_next()
	return file_names

func get_dir_files_with_suffix(directory_path : String, suffix : String):
	var files : Array = get_dir_files(directory_path)
	var result := []
	for file in files:
		if file.ends_with(suffix):
			result.push_back(file)
	return result
